@@ -26,6 +26,7 @@ __date__      = "$Date: $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
+import orca.orca as orca
 import orca.atspi as atspi
 import orca.debug as debug
 import orca.default as default
@@ -40,7 +41,6 @@ import orca.Gecko as Gecko
 
 from orca.orca_i18n import _
 
-
 ########################################################################
 #                                                                      #
 # Custom BrailleGenerator                                              #
@@ -53,17 +53,15 @@ class BrailleGenerator(braillegenerator.BrailleGenerator):
 
     def __init__(self, script):
         self.debugLevel = debug.LEVEL_FINEST
-        
+
         self._debug("__init__")
         braillegenerator.BrailleGenerator.__init__(self, script)
 
-
     def _debug(self, msg):
         """ Convenience method for printing debug messages
         """
         debug.println(self.debugLevel, "Thunderbird.BrailleGenerator: "+msg)
 
-
     def _getDefaultBrailleRegions(self, obj):
         """Gets text to be displayed for the current object's name,
         role, and any accelerators.  This is usually the fallback
@@ -80,7 +78,7 @@ class BrailleGenerator(braillegenerator.BrailleGenerator):
 
         self._debug("_getDefaultBrailleRegions: name='%s', role='%s'" % \
                     (obj.name, obj.role))
-        
+
         regions = []
         text = ""
 
@@ -93,19 +91,21 @@ class BrailleGenerator(braillegenerator.BrailleGenerator):
             # for some labels.
             #
             # Returns whether to consume the event.
-            
+
             self._debug("_getDefaultBrailleRegions: childCount=%d, index=%d" % \
                         (obj.parent.childCount, obj.index))
-            
+
             if obj.index > 0:
                 prev = obj.parent.child(obj.index - 1)
-                self._debug("_getDefaultBrailleRegions: prev='%s', role='%s'" \
-                            % (prev.name, prev.role))
-                
+                if prev:
+                    self._debug("_getDefaultBrailleRegions: prev='%s', role='%s'" \
+                                % (prev.name, prev.role))
+
                 if obj.parent.childCount > obj.index:
                     next = obj.parent.child(obj.index + 1)
-                    self._debug("_getDefaultBrailleRegions: next='%s', role='%s'" \
-                                % (next.name, next.role))
+                    if next:
+                        self._debug("_getDefaultBrailleRegions: next='%s', role='%s'" \
+                                    % (next.name, next.role))
 
             # Get the entry text.
             [word, startOffset, endOffset] = obj.text.getTextAtOffset(0,
@@ -117,7 +117,7 @@ class BrailleGenerator(braillegenerator.BrailleGenerator):
                     atspi.Accessibility.TEXT_BOUNDARY_CHAR)
 
             self._debug("_getDefaultBrailleRegions: word='%s'" % word)
-            
+
             # Determine the order for speaking the component parts.
             if len(word) > 0:
                 if prev and prev.role == rolenames.ROLE_LABEL:
@@ -130,13 +130,13 @@ class BrailleGenerator(braillegenerator.BrailleGenerator):
                         text = _("%s text %s %s") % (obj.name, word, next.name)
                     else:
                         text = _("text %s %s") % (word, obj.name)
-            
+
         else: # non-text object
             text = util.appendString(text, util.getDisplayedLabel(obj))
             text = util.appendString(text, util.getDisplayedText(obj))
             text = util.appendString(text, self._getTextForValue(obj))
             text = util.appendString(text, self._getTextForRole(obj))
-            
+
         self._debug("_getDefaultBrailleRegions: text='%s'" % text)
 
         regions = []
@@ -146,6 +146,24 @@ class BrailleGenerator(braillegenerator.BrailleGenerator):
         return [regions, componentRegion]
 
 
+    def _getBrailleRegionsForText(self, obj):
+        """Get the braille for a text component.
+
+        Arguments:
+        - obj: the text component
+
+        Returns a list where the first element is a list of Regions to display
+        and the second element is the Region which should get focus.
+        """
+
+        # Gecko._getBrailleRegionsForText does not return the correct
+        # braille for Thunderbird. Let the default braillegenerator
+        # handle this.
+        return braillegenerator.BrailleGenerator._getBrailleRegionsForText(
+                self, obj)
+
+
+
 ########################################################################
 #                                                                      #
 # Custom SpeechGenerator for Thunderbird                               #
@@ -163,14 +181,12 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
 
         self._debug("__init__")
         speechgenerator.SpeechGenerator.__init__(self, script)
-        
 
     def _debug(self, msg):
         """ Convenience method for printing debug messages
         """
         debug.println(self.debugLevel, "Thunderbird.SpeechGenerator: "+msg)
 
-
     def _getSpeechForAlert(self, obj, already_focused):
         """Gets the title of the dialog and the contents of labels inside the
         dialog that are not associated with any other objects.
@@ -203,9 +219,8 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
             utterances.append(label.name)
 
         self._debug("unrelated labels='%s'" % utterances)
-        
-        return utterances
 
+        return utterances
 
     def getSpeechContext(self, obj, stopAncestor=None):
         """Get the speech that describes the names and role of
@@ -251,13 +266,6 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
                 parent = parent.parent
                 continue
 
-            # Well...if we made it this far, we will now append the
-            # role, then the text, and then the label.
-            #
-            if parent.role != rolenames.ROLE_TABLE_CELL:
-                self._debug("gettingSpeechForRole: '%s'" % parent.role)
-                utterances.append(rolenames.getSpeechForRoleName(parent))
-
             # Now...autocompletes are wierd.  We'll let the handling of
             # the entry give us the name.
             #
@@ -283,7 +291,6 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
 
         return utterances
 
-
 ########################################################################
 #                                                                      #
 # The Thunderbird script class.                                        #
@@ -307,7 +314,6 @@ class Script(Gecko.Script):
 
         Gecko.Script.__init__(self, app)
 
-
     def getBrailleGenerator(self):
         """Returns the braille generator for this script.
         """
@@ -318,31 +324,49 @@ class Script(Gecko.Script):
         """
         return SpeechGenerator(self)
 
-
     def _debug(self, msg):
         """ Convenience method for printing debug messages
         """
         debug.println(self.debugLevel, "Thunderbird.py: "+msg)
 
-
     def onFocus(self, event):
         """ Called whenever an object gets focus.
 
         Arguments:
         - event: the Event
-        
+
         """
         obj = event.source
+        parent = obj.parent
         top = util.getTopLevel(obj)
         consume = False
 
-        self._debug("onFocus: name='%s', role='%s'" \
-                    % (obj.name, obj.role))
+        self._debug("onFocus: name='%s', role='%s'" % (obj.name, obj.role))
 
-        # Don't speak a chrome URL.
+        # Let onTextInsertion handle autocompletion.
+        if parent.role == rolenames.ROLE_AUTOCOMPLETE:
+            return
+
+        # Don't speak chrome URLs.
         if obj.name.startswith(_("chrome://")):
             return
 
+        # Set focus to the cell at the beginning of the row, so Braille
+        # shows the row from the beginning. Subtract one from the index
+        # because the first child in the table is a list of header names.
+        if obj.role == rolenames.ROLE_TABLE_CELL:
+            table = parent.table
+            inColumn = (obj.index - 1) % table.nColumns
+            if (inColumn != 0):
+                newIndex = obj.index - inColumn
+                self._debug("cell index='%d, inColumn='%d', newIndex='%d'" % \
+                            (obj.index, inColumn, newIndex))
+                # Calling orca.setLocusOfFocus for the first table
+                # cell does not result in the first table cell
+                # being displayed. It's necessary to change the
+                # object index.
+                obj.index = newIndex;
+
         # Handle dialogs.
         if top.role == rolenames.ROLE_DIALOG:
 
@@ -353,23 +377,69 @@ class Script(Gecko.Script):
 
         if not consume:
             Gecko.Script.onFocus(self, event)
-            
+
+    def onStateChanged(self, event):
+        """Called whenever an object's state changes.
+
+        Arguments:
+        - event: the Event
+        """
+
+        # For now, we'll bypass the Gecko script's desire to
+        # let someone know when a page has started/completed
+        # loading.
+        #
+        default.Script.onStateChanged(self, event)
+
+    def onTextInserted(self, event):
+        """Called whenever text is inserted into an object.
+
+        Arguments:
+        - event: the Event
+        """
+        obj = event.source
+        self._debug("onTextInserted: name='%s', role='%s', parent='%s'" %\
+                    (obj.name, obj.role, obj.parent.role))
+
+        parent = obj.parent
+
+        if parent.role == rolenames.ROLE_AUTOCOMPLETE:
+            speech.stop()
+
+            utterances = []
+
+            [text, caretOffset, startOffset] = util.getTextLineAtCaret(obj)
+
+            self._debug("onTextInserted: text='%s'" % text)
+
+            [word, startOffset, endOffset] = obj.text.getTextAtOffset(0,
+               atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
+
+            self._debug("onTextInserted: getTextAtOffset='%s'" % text)
+
+            utterances.append(text)
+            self._debug("onTextInserted: utterances='%s'" % utterances)
+
+            speech.speakUtterances(utterances)
+
+        else:
+            Gecko.Script.onTextInserted(self, event)
 
     def _speakEnclosingPanel(self, obj):
         # Speak the enclosing panel if it is named. Going two
         # containers up the hierarchy appears to be far enough
         # to find a named panel, if there is one.
-        
+
         self._debug("_speakEnclosingPanel")
 
         parent = obj.parent
         if not parent:
             return
-        
+
         if parent.name != "" and \
                (not parent.name.startswith(_("chrome://"))) and \
                parent.role == rolenames.ROLE_PANEL:
-            
+
             # Speak the panel name only once.
             if parent.name != self._containingPanelName:
                 self._containingPanelName = parent.name
@@ -377,14 +447,14 @@ class Script(Gecko.Script):
                 text = _("%s panel") % parent.name
                 utterances.append(text)
                 speech.speakUtterances(utterances)
-            
+
         else:
             grandparent = parent.parent
             if grandparent and \
                    grandparent.name != "" and \
                    (not grandparent.name.startswith(_("chrome://"))) and \
                    grandparent.role == rolenames.ROLE_PANEL:
-                
+
                 # Speak the panel name only once.
                 if grandparent.name != self._containingPanelName:
                     self._containingPanelName = grandparent.name
@@ -393,7 +463,6 @@ class Script(Gecko.Script):
                     utterances.append(text)
                     speech.speakUtterances(utterances)
 
-                        
     def _handleTextEntry(self, obj):
         # Handle preferences that contain editable text fields. If
         # the object with keyboard focus is editable text field,
@@ -401,26 +470,34 @@ class Script(Gecko.Script):
         # for speaking the preference objects.
         #
         # Returns whether to consume the event.
-        
-        self._debug("_handleTextEntry: childCount=%d, index=%d" % \
-                    (obj.parent.childCount, obj.index))
 
-        if not obj.text:
-            return False
-        
+        self._debug("_handleTextEntry: childCount=%d, index=%d, caretOffset=%d, length=%d" % \
+                    (obj.parent.childCount, obj.index, obj.text.caretOffset, obj.text.characterCount))
+
         if obj.index > 0:
             prev = obj.parent.child(obj.index - 1)
-            self._debug("_handleTextEntry: prev='%s', role='%s'" \
-                    % (prev.name, prev.role))
-            
-        if obj.parent.childCount > obj.index:
+            if prev:
+                self._debug("_handleTextEntry: prev='%s', role='%s'" \
+                            % (prev.name, prev.role))
+
+        if obj.index <= obj.parent.childCount - 1:
             next = obj.parent.child(obj.index + 1)
-            self._debug("_handleTextEntry: next='%s', role='%s'" \
-                    % (next.name, next.role))
+            if next:
+                self._debug("_handleTextEntry: next='%s', role='%s'" \
+                            % (next.name, next.role))
 
         # Get the entry text.
         [word, startOffset, endOffset] = obj.text.getTextAtOffset(0,
             atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
+
+        if len(word) == 0:
+            [word, startOffset, endOffset] = obj.text.getTextAtOffset(obj.text.caretOffset,
+                atspi.Accessibility.TEXT_BOUNDARY_LINE_START)
+
+        if len(word) == 0:
+            [word, startOffset, endOffset] = obj.text.getTextAtOffset(0,
+                atspi.Accessibility.TEXT_BOUNDARY_WORD_START)
+
         if len(word) == 0:
             # The above may incorrectly return an empty string
             # if the entry contains a single character.
@@ -441,7 +518,7 @@ class Script(Gecko.Script):
                     text = _("%s text %s %s") % (obj.name, word, next.name)
                 else:
                     text = _("text %s %s") % (word, obj.name)
-                    
+
             speech.speakUtterances([text])
             return True
 
