@@ -21,9 +21,9 @@
 fallback to local settings if the user settings doesn't exist (e.g., in the
 case of gdm) or doesn't have the specified attribute."""
 
-__id__        = "$Id: settings.py 1818 2006-12-13 01:21:31Z wwalker $"
-__version__   = "$Revision: 1818 $"
-__date__      = "$Date: 2006-12-12 20:21:31 -0500 (Tue, 12 Dec 2006) $"
+__id__        = "$Id: settings.py 1996 2007-02-11 18:07:28Z joanied $"
+__version__   = "$Revision: 1996 $"
+__date__      = "$Date: 2007-02-11 13:07:28 -0500 (Sun, 11 Feb 2007) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -445,14 +445,6 @@ debugEventQueue         = False
 timeoutTime             = 10   # a value of 0 means don't do hang checking
 timeoutCallback         = None # Set by orca.py:init to orca.timeout
 
-# Assists with dealing with CORBA COMM_FAILURES.  A failure doesn't
-# always mean an object disappeared - there just might be a network
-# glitch.  So, on COMM_FAILURES, we might retry a few times before
-# giving up on an object.
-#
-commFailureWaitTime     = 0.1
-commFailureAttemptLimit = 5
-
 # Keyboard double-click period. If the same key is pressed within
 # this time period, it's considered to be a double-click and might
 # provide different functionality (for example, Numpad 5 double-click
